__doc__='''
Filesystem session manager.

  Simple enough to be used as an external method. Just add it in the
  External folder inside the Zope distribution and create an external
  method, for instance startSession which points to the startSession
  function of this module. Make sure you create a directory name
  FSSession in your var directory with appropriate permissions.

$Id: FSSession.py,v 1.1 1999/11/28 03:58:57 pavlos Exp pavlos $'''

__version__='$Revision: 1.1 $'[11:-2]


import os,string,time
import marshal
import App.FindHomes
from UserDict import UserDict

FSSessionDir=os.path.join(INSTANCE_HOME,'var','FSSession')


def startSession(self,cookie=1,new=0):
    self.REQUEST['FSSession']=FSStore(self.REQUEST,cookie=cookie,new=new)
    return None



class Store(UserDict):
    '''Implements storage object which acts like a python dictionary'''


    def __getitem__(self,key):
        return self.data.get(key)
    
    get=__getitem__

    def __setitem__(self,key,value):
        self.data[key]=value
        self.dump()

    set=__setitem__

    def __delitem__(self,key):
        del self.data[key]
        self.dump()

    delete=__delitem__

    def clear(self):
        self.data.clear()
        self.dump()

    def copy(self):
        # copy doesn't make much sense in this context
        raise AttributeError,'Copy not supported for Session storage objects'

    def update(self, other):
        if type(other) is type(self.data):
            self.data.update(other)
            self.dump()
        else:
            for k, v in other.items():
                self.data[k] = v
            self.dump()

    def __str__(self):
        '''Pretty print contents'''
        out='<H2>FSSession Contents:</H2>\n'
        for k,v in self.items():
            out='%s\t<LI>%s:\t%s\n'%(out,k,str(v))
        return out
    
class FSStore(Store):
    def __init__(self,request,cookie,new):
        Store.__init__(self)
        self.request=request
        self.cookie=cookie
        if new:
            self.clear()
        else:
            self.load()
            
    def cookie_on(self):
        '''Tracks SessionUID by setting a cookie on the client'''
        self.cookie=1

    def cookie_off(self):
        '''Disables cookie tracking'''
        self.cookie=0

    def load(self,jn=os.path.join):
        if self.request.has_key('FSSessionUID'):
            uid=self.request['FSSessionUID']
        else:
            return None
        day=jn(FSSessionDir,uid[:3])
        hour=jn(day,uid[3:5])
        minute=jn(hour,uid[5:7])

        f=open(jn(minute,uid),'r')
        self.data=marshal.load(f)
        f.close()
        return None

    def dump(self,tm=time,ex=os.path.exists,jn=os.path.join):

        # Create UID
        ip=self.request['REMOTE_ADDR']
        t=tm.gmtime(tm.time())
        # cookie expires after one year
        t_expires=tm.strftime('%a,%d-%b-%Y 23:59:59 GMT',(t[0]+1,)+t[1:])  
        uid='%s%s'%(tm.strftime('%j%I%M%S',t),ip[-5:])
        day=jn(FSSessionDir,uid[:3])
        hour=jn(day,uid[3:5])
        minute=jn(hour,uid[5:7])
    
        if not ex(day):
            os.mkdir(day)
        if not ex(hour):
            os.mkdir(hour)
        if not ex(minute):
            os.mkdir(minute)

        f=open(jn(minute,uid),'w')
        marshal.dump(self.data,f)
        f.close()
        self.request['FSSessionUID']=uid
        if self.cookie:
            self.request.RESPONSE.setCookie('FSSessionUID',
                                            uid,expires=t_expires)
        return None
        
    def sessionURL(self):
        if self.request.has_key('FSSessionUID'):
            return 'FSSessionUID=%s'%self.request['FSSessionUID']
        else:
            return ''







